﻿import org.papervision3d.cameras.*;
import org.papervision3d.view.*;
import org.papervision3d.render.*;
import org.papervision3d.scenes.*;
import org.papervision3d.core.proto.*;
import org.papervision3d.core.view.*;
import org.papervision3d.lights.*;
import org.papervision3d.materials.*;
import org.papervision3d.materials.shadematerials.*;
import org.papervision3d.materials.shaders.*;
import org.papervision3d.materials.utils.*;
import org.papervision3d.objects.*;
import org.papervision3d.objects.primitives.*;

var _height:Number;
var _width:Number;
var camera:CameraObject3D;
var scene:Scene3D;
var viewport:Viewport3D;
var renderer:BasicRenderEngine;

function init3D(viewportWidth:Number = 640, viewportHeight:Number = 480, scaleToStage:Boolean = true, interactive:Boolean = false, cameraType:String = "Target")
{	
	scene = new Scene3D();
	viewport = new Viewport3D(viewportWidth, viewportHeight, scaleToStage, interactive);
	addChild(viewport);
	renderer = new BasicRenderEngine();
	
	switch(cameraType)
	{
		case CameraType.DEBUG:
			camera = new DebugCamera3D(viewport);
			break;
		case CameraType.TARGET:
			camera = new Camera3D(60);
			camera.target = DisplayObject3D.ZERO;
			break;
		case CameraType.FREE:
		default:
			camera = new Camera3D(60);
			break;
	}
	
	//cameraAsCamera3D.update(viewport.sizeRectangle);
	Camera3D(camera).update(viewport.sizeRectangle);
}

/**
 * Exposes the camera as a <code>Camera3D</code>
 */
//function get cameraAsCamera3D():Camera3D
//{
//       return camera as Camera3D;
//}

/**
 * Exposes the camera as a <code>DebugCamera3D</code>
 */
//function get cameraAsDebugCamera3D():DebugCamera3D 
//{
//       return camera as DebugCamera3D;
//}


function startRendering():void
{
	addEventListener(Event.ENTER_FRAME, onRenderTick);
	viewport.containerSprite.cacheAsBitmap = false;
}

function stopRendering(reRender:Boolean = false, cacheAsBitmap:Boolean = false):void
{
	removeEventListener(Event.ENTER_FRAME, onRenderTick);
	if(reRender){
		onRenderTick();	
	}
	if(cacheAsBitmap){
		viewport.containerSprite.cacheAsBitmap = true;
	}else{
		viewport.containerSprite.cacheAsBitmap = false;
	}
}

function singleRender():void
{
	onRenderTick();
}

function onRenderTick(event:Event = null):void
{
	renderer.renderScene(scene, camera, viewport);
}

function set viewportWidth(width:Number):void
{
	_width = width;
	viewport.width = width;
}

function get viewportWidth():Number
{
	return _width;
}

function set viewportHeight(height:Number):void
{
	_height = height;
	viewport.height = height;
}

function get viewportHeight():Number
{
	return _height;
}